# -*- coding: utf-8 -*-
#
# Copyright 2018-2022 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Creating external connectors

"""

from .world import world, teardown_fn, setup_module


class TestCleaningProject(object):

    def setup(self):
        """
            Debug information
        """
        print("\nFinal cleaning\n")

    def test_final(step):
        """Final empty test """
        assert True

    def teardown(self):
        """
            Debug information
        """
        teardown_fn(force=True)
        print("\nEnd of tests.")
